/*********************************************************************
 *              Flowcode Stepper Motor Component Code
 *
 * File: Stepper_Code.c
 *
 * (c) 2007 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 180707 | BR | Created
 * 011008 | SK | Multiple instances enabled. Unipolar patterns added
 *        |    |
 *        |    |
 *        |    |
 *
 *
 *
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   char
   short
   char*

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

 Macro substitutions:
   %a = ??
   %b = ??
   %c = ??
   %d = ??
   %e = ??
   %f = ??
   %g = ??
   %h = ??
   %i = ??
   %j = ??

**********************************************************************

[Settings]
CLSID={}
IsAnalogue=0
MultipleAllowed=1
Description=Stepper Motor Component

**********************************************************************

[Port]
DesiredPort=0
PortMandatory=0
SamePortAsID=0

[Pins]
Count=4
1=Phase 1
2=Phase 2
3=Phase 3
4=Phase 4

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4
1=0
2=0
3=0
4=0

[PinDesiredBit]
1=0
2=1
3=2
4=3

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3
1=0
2=0
3=0
4=0

[PinMustUsePort]
# char
1=1
2=1
3=1
4=1

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number
1=-1
2=-1
3=-1
4=-1

[PinMustUsePin]
# char
1=1
2=1
3=1
4=1

[PinValue]
# char (0 = off, 1 = on)
1=0
2=0
3=0
4=0

**********************************************************************

[MacroNames]
Count=5
1=GetDefines
2=IncrementStep
3=DecrementStep
4=EnableMotor
5=DisableMotor

[MacroReturns]
1=void
2=void
3=void
4=void
5=void

[MacroIsPrivate]
1=1
2=0
3=0
4=0
5=0

[MacroParameters_GetDefines]
Count=0

[MacroParamTypes_GetDefines]

[MacroParameters_IncrementStep]
Count=0

[MacroParamTypes_IncrementStep]

[MacroParameters_DecrementStep]
Count=0

[MacroParamTypes_DecrementStep]

[MacroParameters_EnableMotor]
Count=0

[MacroParamTypes_EnableMotor]

[MacroParameters_DisableMotor]
Count=0

[MacroParamTypes_DisableMotor]

********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*InitialisationCode_Start*/
/*InitialisationCode_End*/

/*InterruptCode_Start*/
/*InterruptCode_End*/




/********************************************************************
 * FUNCTIONS
 ********************************************************************/


void GetDefines()
{
/*Macro_GetDefines_Start*/

}  //Dummy end of function to allow defines to be added correctly

//common defines
#define MX_%q_PORT1			%a	//0
#define MX_%q_TRIS1			%b
#define MX_%q_PIN1			%c
#define MX_%q_PORT2			%d	//1
#define MX_%q_TRIS2			%e
#define MX_%q_PIN2			%f
#define MX_%q_PORT3			%g	//2
#define MX_%q_TRIS3			%h
#define MX_%q_PIN3			%i
#define MX_%q_PORT4			%j	//3
#define MX_%q_TRIS4			%k
#define MX_%q_PIN4			%l
#define MX_%q_STEP_TYPE		%m	//2 = full step, 1 = half step
#define MX_%q_WINDING_TYPE	%n	//0 = bipolar, 1 = unipolar
#define MX_%q_STEPSPERREV	%o
#define MX_%q_STEPRATE		%p

#if (MX_%q_WINDING_TYPE == 0)
	char %q_pattern[8] = {0x05, 0x01, 0x09, 0x08, 0x0a, 0x02, 0x06, 0x04};
#else
	char %q_pattern[8] = {0x01, 0x03, 0x02, 0x06, 0x04, 0x0c, 0x08, 0x09};
#endif

char %q_phase = 0;

// Dummy function to close the defines section off
void %q_Dummy_Function();
void %q_Dummy_Function()
{

/*Macro_GetDefines_End*/
}


void IncrementStep()
{
/*Macro_IncrementStep_Start*/
	char pattern_now;

	%q_phase = (%q_phase + MX_%q_STEP_TYPE) & 0x07;
	pattern_now = %q_pattern[%q_phase];

	if(test_bit(pattern_now, 0)) set_bit (MX_%q_PORT1, MX_%q_PIN1);
	else clear_bit (MX_%q_PORT1, MX_%q_PIN1);
	if(test_bit(pattern_now, 1)) set_bit (MX_%q_PORT2, MX_%q_PIN2);
	else clear_bit (MX_%q_PORT2, MX_%q_PIN2);
	if(test_bit(pattern_now, 2)) set_bit (MX_%q_PORT3, MX_%q_PIN3);
	else clear_bit (MX_%q_PORT3, MX_%q_PIN3);
	if(test_bit(pattern_now, 3)) set_bit (MX_%q_PORT4, MX_%q_PIN4);
	else clear_bit (MX_%q_PORT4, MX_%q_PIN4);
/*Macro_IncrementStep_End*/
}


void DecrementStep()
{
/*Macro_DecrementStep_Start*/
	char pattern_now;

	%q_phase = (%q_phase - MX_%q_STEP_TYPE) & 0x07;
	pattern_now = %q_pattern[%q_phase];

	if(test_bit(pattern_now, 0)) set_bit (MX_%q_PORT1, MX_%q_PIN1);
	else clear_bit (MX_%q_PORT1, MX_%q_PIN1);
	if(test_bit(pattern_now, 1)) set_bit (MX_%q_PORT2, MX_%q_PIN2);
	else clear_bit (MX_%q_PORT2, MX_%q_PIN2);
	if(test_bit(pattern_now, 2)) set_bit (MX_%q_PORT3, MX_%q_PIN3);
	else clear_bit (MX_%q_PORT3, MX_%q_PIN3);
	if(test_bit(pattern_now, 3)) set_bit (MX_%q_PORT4, MX_%q_PIN4);
	else clear_bit (MX_%q_PORT4, MX_%q_PIN4);
/*Macro_DecrementStep_End*/
}


void EnableMotor()
{
/*Macro_EnableMotor_Start*/
	clear_bit(MX_%q_TRIS1, MX_%q_PIN1);
	clear_bit(MX_%q_PORT1, MX_%q_PIN1);
	clear_bit(MX_%q_TRIS2, MX_%q_PIN2);
	clear_bit(MX_%q_PORT2, MX_%q_PIN2);
	clear_bit(MX_%q_TRIS3, MX_%q_PIN3);
	clear_bit(MX_%q_PORT3, MX_%q_PIN3);
	clear_bit(MX_%q_TRIS4, MX_%q_PIN4);
	clear_bit(MX_%q_PORT4, MX_%q_PIN4);
/*Macro_EnableMotor_End*/
}


void DisableMotor()
{
/*Macro_DisableMotor_Start*/
	set_bit(MX_%q_TRIS1, MX_%q_PIN1);
	clear_bit(MX_%q_PORT1, MX_%q_PIN1);
	set_bit(MX_%q_TRIS2, MX_%q_PIN2);
	clear_bit(MX_%q_PORT2, MX_%q_PIN2);
	set_bit(MX_%q_TRIS3, MX_%q_PIN3);
	clear_bit(MX_%q_PORT3, MX_%q_PIN3);
	set_bit(MX_%q_TRIS4, MX_%q_PIN4);
	clear_bit(MX_%q_PORT4, MX_%q_PIN4);
/*Macro_DisableMotor_End*/
}

